<?php

/**
 * @file
 * Contains the code to generate the mask_user_data drush commands.
 */

use Drupal\user\Entity\User;

/**
 * Implements hook_drush_command().
 */
function mask_user_data_drush_command() {
  $items = [];

  $items['mask_user_data'] = [
    'description' => dt('Masks user data in the database.'),
    'aliases' => ['mud'],
    'examples' => [
      'drush mud' => 'Masks user data in the database.',
    ],
  ];

  return $items;
}

/**
 * Drush callback to mask user data.
 */
function drush_mask_user_data() {
  $enabled = Drupal::config('mask_user_data.settings')->get('enabled') ?: FALSE;
  if ($enabled) {
    drush_log(dt('Data masking is enabled.'), 'success');

    $already_masked = Drupal::config('mask_user_data.settings')->get('already_masked') ?: FALSE;
    if ($already_masked) {
      drush_log(dt('The user data has already been masked.'), 'success');
    }
    else {
      if (drush_confirm('Are you sure you want to mask user data?')) {
        $map_array = Drupal::config('mask_user_data.settings')->get('map_array') ?: NULL;
        if (empty($map_array) || !is_array($map_array)) {
          drush_log(dt('Invalid or empty map array. Please check the configuration.'), 'error');
        }
        else {
          mask_user_data_setup_batch();
        }
      }
      else {
        drush_user_abort();
      }
    }
  }
  else {
    drush_log(dt('Data masking is disabled.'), 'success');
  }
}

/**
 * Sets up a batch to process all users in the system and mask their data.
 *
 * @see https://www.metaltoad.com/blog/using-drupal-batch-api
 */
function mask_user_data_setup_batch() {
  $all_users = Drupal::database()
    ->query("SELECT uid FROM users WHERE uid > 0")
    ->fetchCol();

  // Break up all of our data so each process does not time out.
  $chunks = array_chunk($all_users, 20);
  $operations = [];
  $count_chunks = count($chunks);

  // For every chunk, assign some method to run on that chunk of data.
  $i = 0;
  foreach ($chunks as $chunk) {
    $i++;
    $operations[] = [
      "mask_user_data_mask_users",
      [
        $chunk,
        t(
          '(Masking users: chunk @chunk  of @count)',
          [
            '@chunk ' => $i,
            '@count' => $count_chunks,
          ]
        ),
      ],
    ];
  }

  // Put all that information into our batch array.
  $batch = [
    'operations' => $operations,
    'title' => t('Import batch'),
    'init_message' => t('Initializing'),
    'error_message' => t('An error occurred'),
    'finished' => 'mask_user_data_finished_method',
  ];

  // Let's run the batch now.
  // Because we are doing this on the back-end, we set progressive to false.
  drush_log(dt('Updating all the fields and properties of the existing users. This may take some time.'), 'success');
  batch_set($batch);
  $batch =& batch_get();
  $batch['progressive'] = FALSE;
  drush_backend_batch_process();
}

/**
 * Masks a group of users.
 *
 * @param array $chunk
 *   List of uids.
 * @param string $operation_details
 *   Message.
 * @param object $context
 *   Operation context.
 */
function mask_user_data_mask_users(array $chunk, $operation_details, &$context) {
  $map_array = Drupal::config('mask_user_data.settings')->get('map_array') ?: NULL;
  $user = User::load(1);
  $user_fields = $user->getFields();

  foreach ($chunk as $uid) {
    Drupal::service('mask_user_data.mask_user')->mask($uid, $map_array, $user_fields);
  }

  $context['message'] = $operation_details;
}

/**
 * Callback function when the batch completes.
 *
 * @param mixed $success
 *   Success information.
 * @param mixed $results
 *   Results information.
 * @param mixed $operations
 *   Operations information.
 */
function mask_user_data_finished_method($success, $results, $operations) {
  Drupal::configFactory()->getEditable('mask_user_data.settings')->set('already_masked', TRUE)->save();
  drush_log(dt('Update completed, all data has been masked.'), 'success');
}
